/*

 Copyright (C) Dirk Haase
 Permission is granted to any individual or institution to use, copy, or
 redistribute this software so long as all of the original files are included,
 that it is not sold for profit, and that this copyright notice is retained.

*/

#ifndef __MACOS_OSDEP_H
#define __MACOS_OSDEP_H 1

#ifndef MACOS
#  define MACOS
#endif

#include <setjmp.h>
#include <types.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <unix.h>
#include <unistd.h>
#include <console.h>

#include <Errors.h>
#include <Files.h>

#include "unixlike.h"
#include "macglob.h"


#define NO_MKTEMP           1
#define HAVE_DIRENT_H       1
#define PASSWD_FROM_STDIN   1
#define NO_SYMLINK          1

#define USE_ZIPMAIN         1
#define USE_ZIPSPLITMAIN    1
#define USE_ZIPNOTEMAIN     1

#ifdef UTIL
int     zipsplitmain(int argc,char **argvalue);
int     zipnotemain(int argc,char **argvalue);
#endif



/*   
#define DEBUG_TIME 
 */

#if (!defined(NO_EF_UT_TIME) && !defined(USE_EF_UT_TIME))
#  define USE_EF_UT_TIME
#endif

#undef  IZ_CHECK_TZ
 
#ifndef ZP_NEED_MEMCOMPR
#  define ZP_NEED_MEMCOMPR
#endif


#define EXDEV  18

#define PATHCUT ':'


/* file operations use "b" for binary */
#define FOPR  "rb"
#define FOPM  "r+b"
#define FOPW  "wb"

/*
#define DEBUG
*/

/* These functions are defined as a macro instead of a function.
so we have to undefine them for replacing (see printf.c)  */
#undef getc
#undef getchar
#undef putchar



void setfiletype(char *file, unsigned long Creator, unsigned long Type);

char *GetZipVersionsInfo(void);
char *GetZipVersionLocal(void);
char *GetZipCopyright(void);

void InitAllVars(void);

void PrintFileInfo(void);





/* Many things are different for mac-users,
   so we need special mac functions :-) 
int MacStat (char *path, struct stat *buf);
#define stat(path, bufPtr) MacStat(path, bufPtr)
#define lstat(path, bufPtr) MacStat(path, bufPtr) */

int fprintf(FILE *file, const char *format, ...);
int printf(const char *format, ...);
void perror(const char *parm1);
int macgetch(void);


int MacOpen(const char *path,int oflag,...);
FILE *MacFopen(const char *path,const char *mode);
#define fopen(path, mode)   MacFopen(path, mode)
#define open(path, oflag)   MacOpen(path, oflag)


char *GetComment(char *filename);
int readlink(char *path, char *buf, int size);

void PrintStatProgress(char *msg);


/*
The following macro transforms this statement e.g:
if ((r = PROCNAME(argv[i])) != ZE_OK)

into this statement:
if ((r = (action == 1 || action == 2 ? wild(argv[i]) : procname(argv[i], 1))) != 0)

Just to make it clear :-)
The zip.c calls now (if this macro is active) also the function wild(char *)

*/

#define PROCNAME(n) (action == ADD || action == UPDATE ? wild(n) : \
                     procname(n, 1))

#endif   /* __MACOS_OSDEP_H */
